; > monbbc.asm
;
; Include for for EhBasic for BBC target
; 10-Oct-2016 J.G.Harston
; 06-Sep-2021 Also claim VEC_CC to test Escape without INKEY
;
; This can be built with cc65 with:
; cl65 -t none -l --feature labels_without_colons -o ehbasic.bin monbbc.asm 2> ehbasic.log


.include "ver.asm"
.define	 TARGET	BBC

BREAK    =$1B		; Code of keypress to interrupt program
MEMBOT   =0		; Ask MOS for bottom of memory calling MON_LIMITS
MEMTOP   =0		; Ask MOS for top of memory calling MON_LIMITS
ZPSTART  =$00		; Start of zero page workspace
ZPWARM   =$00		; Warm start at &0000
ZPEND    =$8F		; End of zero page workspace
ZPROUT   =0		; No subroutines in zero page
WSSTART  =$0400		; Start of vector and input buffer workspace
.ifndef CODESTART
CODESTART=$C000		; Start of program code
.endif

.ORG	CODESTART
	JMP BBC_START
	.byte 0,0,0
	.byte $62		; type=6502 code
	.byte Copyright-CODESTART
	.byte $00
	.byte "EhBASIC"
	.byte $00
	.byte VERSION
Copyright
	.byte 0,"(C)Lee Davison",0
	.word CODESTART
	.word 0
BBC_START
	LDX	#$FF
	TXS			; Clear stack
	CLD			; Ensure binary mode
	CLI			; Ensure interupts enabled
	BCS	NOTRESET
	LDA	#$0B
	JSR	$FFEE
	JSR	$FFEE
NOTRESET


.INCLUDE "ehbasic.asm"		; ** INCLUDE MAIN BASIC CODE **


MON_INIT
	LDA	#<BBC_ERROR	; Claim error vector
	STA	$0202
	LDA	#>BBC_ERROR
	STA	$0203

	LDX	#VECEND-VECSTART
HOSTLP1
	LDA	VECSTART,X	; Set up vectors
;	STA	VEC_IN,X
	STA	VEC_CC,X
	DEX
	BPL	HOSTLP1
	RTS

VECSTART
	.word	BBCESCCHK
	.word	BBCRDCH
	.word	$FFEE		; BBCWRCH
	.word	BBCLOAD
	.word	BBCSAVE
	.word	BBCRTS
VECEND

MON_LIMITS
	LDA	#$84
	JSR	$FFF4		; Ask MOS for top of memory
	STX	Ememl		; set end of mem low byte
	STY	Ememh		; set end of mem high byte
	STX	Sstorl		; set bottom of string space low byte
	STY	Sstorh		; set bottom of string space high byte
	LDA	#$83
	JSR	$FFF4
	TXA
	BNE	MON_LIM2
	CPY	#$09
	BCS	MON_LIM2	; If MEMBOT runs on from language workspace,
	LDY	#$05		;  use language workspace for program
MON_LIM2
	STX	Smeml		; save start of mem low byte
	STY	Smemh		; save start of mem high byte
	RTS

; Error handler
BBC_ERROR
	LDX	$FD		; XY=>FAULT
	LDY	$FE
	INX
	TXA
	BNE	BBC_ERROR2	; Point to error message
	INY
BBC_ERROR2
	JMP	LAB_ERR2

; Check for Escape without polling keyboard
BBCESCCHK
        JSR     BBCESCPOLL
	BCC     BBCESCIGN	; No Escape
	LDA	#BREAK
	JMP	LAB_1636	; Jump to BREAK handler
        
BBCESCPOLL
        CLC
	LDA	ccflag		; get [CTRL-C] check flag
	BNE	BBCESCIGN	; exit if inhibited
        BIT     $FF
        BPL     BBCESCIGN
	LDA	#$7E
	JSR	$FFF4		; Acknowledge Escape
        SEC
BBCESCIGN
        RTS
        
; Check for keypress
BBCRDCH	PHA
	TYA
	PHA
	TXA
	PHA
        JSR     BBCESCPOLL
	LDY	#$00		; Prepare $001B for Escape
        LDX     #$1B
        BCS     BBCESC		; Pending Escape state
	LDX	#$01		; Poll for $0001 cs
	LDA	#$81
	JSR	$FFF4		; Test for keypress
	INY                     ; No key $FF becomes $00
	CPY	#$01		; CS=key pressed, CC=no key pressed
BBCESC	TXA
	TSX
	STA	$103,X
	PLA
	TAX
	PLA
	TAY
	PLA			; Set EQ from input character
	RTS

;; Output a character
;BBCWRCH
;	JMP	$FFEE

; LOAD and SAVE
BBCLOAD
	LDA	#$FF		; $FF=load
	JSR	BBCFILE
	LDA	#<LAB_RMSG	; "READY"
	LDY	#>LAB_RMSG
	JSR	LAB_18C3
	JMP	LAB_1319	; relink program and clear variables
BBCSAVE
	LDA	#$00		; $00=save
BBCFILE
	PHA			; save operation
	JSR	LAB_EVEX
	JSR	LAB_EVST	; evaluate string
	JSR	BBCSTRG		; convert to <cr>-string
	PLA
	STA	Btmpl		; save operation
	LDX	#$12
	LDA	#$00
BBCFILE2
	PHA			; create control block on stack
	DEX
	BNE	BBCFILE2
	TSX
	INX			; X=>control block on stack
	LDA	Smeml		; start of program
	SEC			; step back to save from initial $00 byte
	SBC	#1		; so binary file recognised by first byte
	STA	$0102,X
	STA	$010A,X
	LDA	Smemh
	SBC	#0
	STA	$0103,X
	STA	$010B,X
	LDA	Svarl		; end of program
	STA	$0106,X
	STA	$010E,X
	LDA	Svarh
	STA	$0107,X
	STA	$010F,X
	LDA	#<Ibuffs	; filename
	STA	$0100,X
	LDA	#>Ibuffs
	STA	$0101,X
	LDA	Btmpl
	BEQ	BBCFILE3	; skip past with SAVE
	LDA	#$00
	STA	$0106,X		; load to supplied address
	LDA	#$FF
BBCFILE3
	LDY	#$01		; XY=>control block
	JSR	$FFDD		; call OSFILE
	TSX
	TXA
	CLC
	ADC	#$12		; drop control block from stack
	TAX
	TXS
	RTS

; Convert EhBasic string to <cr>-string
BBCSTRG				; XY=>string, A=max length
	STX	Btmpl
	STY	Btmph
	LDY	#$FF
	TAX			; X=length
	BEQ	BBCSTRNULL	; null string
BBCSTRLP
	INY
	LDA	(Btmpl),Y	; Convert to <cr>-terminated string
	STA	Ibuffs,Y
	BEQ	BBCSTRCR	; End of string
	DEX
	BNE	BBCSTRLP	; Loop until X=0
BBCSTRNULL
	INY
BBCSTRCR
	LDA	#$0D
	STA	Ibuffs,Y
	LDX	#<Ibuffs
	LDY	#>Ibuffs
BBCRTS
	RTS
